﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.ReportDesigner;
using Microsoft.ReportingServices.Interfaces;
using Microsoft.ReportingServices.RdlObjectModel;
using Microsoft.ReportDesigner.Design;


namespace ixto.Beispiele.ReportingServices
{
    [LocalizedName("myCustomReportItem")]
    //[ToolboxBitmap(@"ixto.ico")]
    [CustomReportItem("myCustomReportItem")]
    
    public class CustomReportItem1Designer : CustomReportItemDesigner
    {
        private IComponentChangeService m_changeSvc;

        #region "Farbe für Durchschnitt-Markierung"
        private Color colorAverage = Color.Black;//Farbe der Markierung
        [ Browsable(true), Category("CRI Darstellung"), Description("Farbe für Durchschnitt-Markierung")]
        public Color DurchschnittFarbe
        {
            get
            {
               return colorAverage;
            }
            set
            {
                SetCustomProperty("DurchschnittFarbe", value.Name);
                colorAverage = value;
                Invalidate();
            }
        }
        #endregion

        #region "Balkenbreite"
        [Browsable(true), Category("CRI Darstellung"), DefaultValue("8")
        , Description("Breite der darzustellenden Elemente.")]
        public string Balkenbreite
        {
            get
            {
               string breite = this.GetCustomProperty("Balkenbreite");
               return string.IsNullOrEmpty(breite) ? "8" : breite;
             }
             set
             {
                SetCustomProperty("Balkenbreite", value);
                Invalidate();
             }
        }
        #endregion

        #region "BalkenFarbe"
        private Color colorCRI = Color.Blue;//Farbe für das CRI
        [Browsable(true), Category("CRI Darstellung"), Description("Farbe der darzustellenden Elemente.")]
        public Color BalkenFarbe
        {
           get
           {
               return colorCRI;
           }
           set
           {
               SetCustomProperty("BalkenFarbe", value.Name);
               colorCRI = value;
               Invalidate();
            }
        }
        #endregion

        #region "Durchschnitt"
        [Browsable(true), Category("CRI Data"), DefaultValue(""), Description("Durchschnitt")]
        public string Durchschnitt
        {
           get
           {
               string avarage = this.GetCustomProperty("Durchschnitt");
               return string.IsNullOrEmpty(avarage) ? "0" : avarage;
           }
           set
           {
               SetCustomProperty("Durchschnitt", value);
               Invalidate();
           }
        }
        #endregion

        #region "Maximum"
        [Browsable(true), Category("CRI Data"), DefaultValue("0"), Description("Maximaler Ausschlag")]
        public string Maximum
        {
           get
           {
               string max = this.GetCustomProperty("Maximum");
               return string.IsNullOrEmpty(max) ? "0" : max;
           }
           set
           {
               SetCustomProperty("Maximum", value);
               Invalidate();
           }
        }
        #endregion

        #region "ValueField"
        [Browsable(true), Category("CRI Data"), DefaultValue("")
        , Description("Name des Datasetfeldes, dessen Werte-Liste verwendet werden soll")]
        public string ValueField
        {
           get
           {
               string valueField = this.GetCustomProperty("ValueField");
               return string.IsNullOrEmpty(valueField) ? "" : valueField;
           }
           set
           {
               SetCustomProperty("ValueField", value);
               Invalidate();
           }
        }
        #endregion

        #region Hilfsfunktionen der Eigenschaften
        //Hilfsfunktion, um Werte aus CustomData (generiert in InitializeNewComponent) auszulesen
        public string GetCustomProperty(string propertyname)
        {
           return CustomProperties.ContainsKey(propertyname) ? CustomProperties[propertyname] : null;
        }
        public void SetCustomProperty(string propertyname, string value)
        {
           if (CustomProperties.ContainsKey(propertyname))
           {
              CustomProperties[propertyname] = value;
           }
           else
           {
              CustomProperties.Add(propertyname, value);
           }
        }
        #endregion

        #region Zeichnen der Grafik
        public override void Draw(Graphics gr, ReportItemDrawParams dp)
        {
           int pixelWidth  = (int)Math.Round(Width);
           int pixelHeight = (int)Math.Round(Height);

           //Erstellung einer Farbe vom Typ Color und Werte für die Berechnung der Grafik
           Color color = colorCRI;
           int balkenbreite;
           balkenbreite = (int)Convert.ToSingle(Balkenbreite);

           #region "Hintergrundfarbe vom Typ SolidBrush"
           SolidBrush backgroundColorBrush = new SolidBrush(Style.BackgroundColor.Value.Color);
           gr.FillRectangle(backgroundColorBrush, 0, 0, pixelWidth, pixelHeight);
           #endregion

           //zentrales Erzeugen eines Pen-Objektes mit benutzerdefinierter Farbe und Breite
           Pen penBalken       = new Pen(colorCRI, balkenbreite);
           Pen penDurchschnitt = new Pen(colorAverage, 1);

           //Positionen für Darstellung der jeweiligen Linien festlegen (Balken und Durchschnitt-Markierung)
           float sampleXOffset = pixelWidth  / 2;
           float sampleYOffset = pixelHeight / 2;

           //Linie im Layoutbereich mit der auswählten Farbe "BalkenFarbe" zeichnen
           gr.DrawLine(penBalken, 0, sampleYOffset, sampleXOffset, sampleYOffset);

           //zeichnen der Linie für den Durchschnitt mit der auswählten Farbe "DurchschnittFarbe"
           gr.DrawLine(penDurchschnitt, sampleXOffset, 1, sampleXOffset, pixelHeight - 1);

           //Freigabe der verwendeten Ressource des zuvor erstellten Objektes
           backgroundColorBrush.Dispose();
        }
        #endregion

        public IComponentChangeService ChangeService()
        {
           if (m_changeSvc == null)
           {
               m_changeSvc = (IComponentChangeService)Site.GetService(typeof(IComponentChangeService));
           }
           return m_changeSvc;
         }
    }       
}
